#pragma once

#include "Common.hpp"
#include "Context.hpp"
#include "ContextManager.hpp"
#include "Debug.hpp"
#include "Runtime.hpp"

template <LogSubsystem logSubsystem>
class Blockable : public Logger<logSubsystem> {
 protected:
	Runtime& runtime;

	ContextManager& contextManager;

	Blockable(Runtime& runtime) : runtime(runtime), contextManager(runtime.getContextManager()) {}

	void block(func_t freshContextHook) {
		LOGD("block() blockedContext is " << Context::getCurrentContext());
		contextManager.saveAndStartNew(freshContextHook);
	}

	void unblock(Context* context) {
		assert(context != nullptr);
		// cppcheck-suppress unsafeClassCanLeak
		Fiber* unblockFiber =
				Fiber::from([this, context]() { contextManager.discardAndResume(context); });
		runtime.schedule(*unblockFiber);
	}
};
