#include "CountingPrivateSemaphore.hpp"

#include <cassert>

CountingPrivateSemaphore::CountingPrivateSemaphore() : CountingPrivateSemaphore(0) {}

CountingPrivateSemaphore::CountingPrivateSemaphore(unsigned int counter)
		: counter(counter), blockedContext(nullptr) {}

void CountingPrivateSemaphore::incrementCounterByOne() { counter++; }

void CountingPrivateSemaphore::incrementCounter(unsigned int count) { counter += count; }

void CountingPrivateSemaphore::wait() {
	if (counter > 0) {
		Context* blockedContext = Context::getCurrentContext();
		block([this, blockedContext] {
			this->blockedContext = blockedContext;
			assert(blockedContext > (Context*)4096);
			if (this->getCounter() == 0) {
				Context* readyContext = this->blockedContext.exchange(nullptr);
				if (readyContext != nullptr) {
					unblock(readyContext);
				}
			}
		});
	}
}

Context* CountingPrivateSemaphore::signalInternal() {
	unsigned int oldCounter = counter.fetch_sub(1);
	assert(oldCounter >= 1);

	// If the counter is still non-zero after the decrement, somebody
	// else is responsible for scheduling the fiber.
	if (oldCounter > 1) return nullptr;

	if (blockedContext.load() != nullptr && counter == 0) {
		// Try to swap out a blocked context, it is fine if this
		// returns nullptr. In this case the block() function will
		// have won the race.
		Context* context = blockedContext.exchange(nullptr);
		assert(context > (Context*)4096);
		return context;
	}

	return nullptr;
}
