#pragma once

#include "Common.hpp"
#include "Debug.hpp"
#include "Fiber.hpp"

class Runtime;
class ContextManager;

class Dispatcher : public Logger<LogSubsystem::DISP> {
 protected:
	static thread_local const Fiber* currentFiber;

	Runtime& runtime;

	virtual void dispatchLoop() = 0;

	func_t getDispatchLoop();

	inline void dispatch(const Fiber* fiber) {
		LOGD("executing fiber " << fiber);
		currentFiber = fiber;
		fiber->run();
	}

	static inline bool isRunnable(Fiber* fiber) { return fiber->setRunnableFalse(); }

	static inline workeraffinity_t* getAffinityBuffer(Fiber* fiber) {
		return fiber->getAffinityBuffer();
	}

	static inline unsigned int decreaseRefCount(Fiber* fiber) {
		return fiber->doAtomicDecrRefCount();
	}

	inline void recycle(const Fiber* fiber) { delete fiber; }

	void putRuntimeWorkerToSleep();

 public:
	Dispatcher(Runtime& runtime) : runtime(runtime) {}

	static const Fiber& getCurrentFiber() {
		const Fiber* fiber = getCurrentFiberPtr();
		return *fiber;
	}

	static const Fiber* getCurrentFiberPtr() {
		assert(currentFiber);
		return currentFiber;
	}

	static bool isDispatchedControlFlow() { return currentFiber != nullptr; }

	friend ContextManager;
};
