#pragma once

#include "Blockable.hpp"
#include "Context.hpp"
#include "ContextManager.hpp"
#include "Debug.hpp"
#include "Fiber.hpp"
#include "Runtime.hpp"

class PrivateSemaphore : protected Blockable<LogSubsystem::PS> {
 protected:
	// cppcheck-suppress uninitMemberVar
	PrivateSemaphore() : Blockable(*Runtime::getRuntime()) {
		LOGD("constructed by fiber " << Dispatcher::getCurrentFiber());
	}

	[[noreturn]] void unblockAndExit(Context* context) { contextManager.discardAndResume(context); }

	virtual Context* signalInternal() = 0;

	virtual void debugLog(const std::string& string) const { Logger::logD(string); }

 public:
	virtual void wait() = 0;

	void signal() {
		if (Context* readyContext = signalInternal()) {
			unblock(readyContext);
		}
		// Assure that nothing happens after here with the sync
		// primitive, as the data of the sync primitive may became
		// invalid at this point.
	}

	void signalAndExit() {
		if (Context* readyContext = signalInternal()) {
			unblockAndExit(readyContext);
		}
		// Assure that nothing happens after here with the sync
		// primitive, as the data of the sync primitive may became
		// invalid at this point.
	}
};

typedef PrivateSemaphore PS;
