#pragma once

#include <mutex>
#include <queue>

#include "BinaryPrivateSemaphore.hpp"

namespace emper {

class Semaphore {
 private:
	std::queue<BinaryPrivateSemaphore*> waiterList;
	unsigned int count;
	std::mutex mutex;

 public:
	bool acquire() {
		bool blocked;
		mutex.lock();
		if (count > 0) {
			count--;
			mutex.unlock();
			blocked = false;
		} else {
			BinaryPrivateSemaphore semaphore;
			waiterList.push(&semaphore);
			mutex.unlock();
			semaphore.wait();
			blocked = true;
		}
		return blocked;
	}

	bool release() {
		mutex.lock();
		bool waiterListEmpty = waiterList.empty();
		if (waiterListEmpty) {
			count++;
			mutex.unlock();
		} else {
			BinaryPrivateSemaphore* semaphore = waiterList.front();
			waiterList.pop();
			mutex.unlock();
			semaphore->signal();
		}
		return waiterListEmpty;
	}

	void print();
};

}	 // namespace emper
