#pragma once

#include "BinaryPrivateSemaphore.hpp"
#include "CountingPrivateSemaphore.hpp"
#include "Fiber.hpp"
#include "PrivateSemaphore.hpp"

class SynchronizedFiber : public Fiber {
 private:
	PrivateSemaphore& semaphore;

	SynchronizedFiber(fiber_fun_t function, void* arg, workeraffinity_t* affinity,
										PrivateSemaphore& semaphore)
			: Fiber(function, arg, affinity), semaphore(semaphore){};

	// cppcheck-suppress uninitMemberVar
	explicit SynchronizedFiber(fiber_fun0_t function, workeraffinity_t* affinity,
														 PrivateSemaphore& semaphore)
			: SynchronizedFiber([function](UNUSED_ARG void* arg) { function(); }, nullptr, affinity,
													semaphore) {}

	SynchronizedFiber(fiber_fun_t function, void* arg, PrivateSemaphore& semaphore)
			: SynchronizedFiber(function, arg, nullptr, semaphore){};

	// cppcheck-suppress uninitMemberVar
	explicit SynchronizedFiber(fiber_fun0_t function, PrivateSemaphore& semaphore)
			: SynchronizedFiber(function, nullptr, semaphore) {}

	void run() const override {
		Fiber::run();
		semaphore.signalAndExit();
	}

 public:
	static inline Fiber* from(fiber_fun_t function, void* arg, BinaryPrivateSemaphore& semaphore) {
		return new SynchronizedFiber(function, arg, semaphore);
	}

	static inline Fiber* from(fiber_fun0_t function, BinaryPrivateSemaphore& semaphore) {
		return new SynchronizedFiber(function, semaphore);
	}

	static inline Fiber* from(fiber_fun_t function, void* arg, workeraffinity_t* affinity,
														BinaryPrivateSemaphore& semaphore) {
		return new SynchronizedFiber(function, arg, affinity, semaphore);
	}

	static inline Fiber* from(fiber_fun0_t function, workeraffinity_t* affinity,
														BinaryPrivateSemaphore& semaphore) {
		return new SynchronizedFiber(function, affinity, semaphore);
	}

	static inline Fiber* from(fiber_fun_t function, void* arg, CountingPrivateSemaphore& semaphore) {
		semaphore.incrementCounterByOne();
		return new SynchronizedFiber(function, arg, semaphore);
	}

	static inline Fiber* from(fiber_fun0_t function, CountingPrivateSemaphore& semaphore) {
		semaphore.incrementCounterByOne();
		return new SynchronizedFiber(function, semaphore);
	}

	static inline Fiber* from(fiber_fun_t function, void* arg, workeraffinity_t* affinity,
														CountingPrivateSemaphore& semaphore) {
		semaphore.incrementCounterByOne();
		return new SynchronizedFiber(function, arg, affinity, semaphore);
	}

	static inline Fiber* from(fiber_fun0_t function, workeraffinity_t* affinity,
														CountingPrivateSemaphore& semaphore) {
		semaphore.incrementCounterByOne();
		return new SynchronizedFiber(function, affinity, semaphore);
	}
};
