#include "BinaryPrivateSemaphore.hpp"
#include "CountingPrivateSemaphore.hpp"
#include "Fiber.hpp"
#include "Runtime.hpp"
#include "emper.h"

runtime* init_runtime(void) {
	Runtime* r = new Runtime();
	return reinterpret_cast<runtime*>(r);
}

void wait_until_runtime_finished() {
	Runtime* r = Runtime::getRuntime();
	r->waitUntilFinished();
}

fiber* fiber_from(void (*function)(void*), void* arg) {
	Fiber* f = Fiber::from(function, arg);
	return reinterpret_cast<fiber*>(f);
}

fiber* fiber_from0(void (*function)(void)) {
	Fiber* f = Fiber::from(function);
	return reinterpret_cast<fiber*>(f);
}

fiber* aff_fiber_from(void (*function)(void*), void* arg, workeraffinity_t* affinity) {
	Fiber* f = Fiber::from(function, arg, affinity);
	return reinterpret_cast<fiber*>(f);
}

fiber* aff_fiber_from0(void (*function)(void), workeraffinity_t* affinity) {
	Fiber* f = Fiber::from(function, affinity);
	return reinterpret_cast<fiber*>(f);
}

void init_affinity(workeraffinity_t affinity[], unsigned int n) {
	for (unsigned int i = 0; i < n; ++i) {
		affinity[i] = Fiber::NOT_AFFINE;
	}
}

void schedule(fiber* fiber) {
	Runtime* runtime = Runtime::getRuntime();
	Fiber* f = reinterpret_cast<Fiber*>(fiber);
	runtime->schedule(*f);
}

bps* new_binary_sem() {
	BPS* sem = new BPS();
	return reinterpret_cast<bps*>(sem);
}

void signal_bps(bps* sem) {
	BinaryPrivateSemaphore* binaryPrivateSemaphore = reinterpret_cast<BinaryPrivateSemaphore*>(sem);
	binaryPrivateSemaphore->signal();
}

void signal_and_exit_bps(bps* sem) {
	BinaryPrivateSemaphore* binaryPrivateSemaphore = reinterpret_cast<BinaryPrivateSemaphore*>(sem);
	binaryPrivateSemaphore->signalAndExit();
}

void wait_bps(bps* sem) {
	BinaryPrivateSemaphore* binaryPrivateSemaphore = reinterpret_cast<BinaryPrivateSemaphore*>(sem);
	binaryPrivateSemaphore->wait();
}

cps* new_counting_sem() {
	CPS* sem = new CPS();
	return reinterpret_cast<cps*>(sem);
}

cps* new_counting_sem_with(unsigned int i) {
	CPS* sem = new CPS(i);
	return reinterpret_cast<cps*>(sem);
}

void signal_cps(cps* sem) {
	CountingPrivateSemaphore* countingPrivateSemaphore =
			reinterpret_cast<CountingPrivateSemaphore*>(sem);
	countingPrivateSemaphore->signal();
}

void signal_and_exit_cps(cps* sem) {
	CountingPrivateSemaphore* countingPrivateSemaphore =
			reinterpret_cast<CountingPrivateSemaphore*>(sem);
	countingPrivateSemaphore->signalAndExit();
}

void wait_cps(cps* sem) {
	CountingPrivateSemaphore* countingPrivateSemaphore =
			reinterpret_cast<CountingPrivateSemaphore*>(sem);
	countingPrivateSemaphore->wait();
}
