#pragma once

#include <deque>
#include <mutex>

#include "Common.hpp"

namespace adt {

template <typename I, const uintptr_t SIZE>
class LockedQueue {
 private:
	std::mutex queue_mutex;

	std::deque<I> deque;

 public:
	bool isFull() const {
		std::lock_guard<std::mutex> lock(queue_mutex);
		return deque.size() == SIZE;
	}

	bool pushBottom(const I item) {
		std::lock_guard<std::mutex> lock(queue_mutex);

		if (deque.size() == SIZE) return false;

		deque.push_back(item);

		return true;
	}

	bool popTop(I* itemPtr) {
		std::lock_guard<std::mutex> lock(queue_mutex);

		if (deque.empty()) return false;

		*itemPtr = deque.front();

		deque.pop_front();

		return true;
	}

	bool popBottom(I* itemPtr) {
		std::lock_guard<std::mutex> lock(queue_mutex);

		if (deque.empty()) return false;

		*itemPtr = deque.back();

		deque.pop_back();

		return true;
	}
};
}	 // namespace adt
