#include "LawsDispatcher.hpp"

#include "LawsStrategy.hpp"
#include "Runtime.hpp"
#include "emper-config.h"

void LawsDispatcher::dispatchLoop() {
	while (true) {
		Fiber* const fiber = runtime.nextFiber();
		if (!fiber) {
#ifdef EMPER_WORKER_SLEEP
			putRuntimeWorkerToSleep();
#else
			pthread_yield();
#endif
			continue;
		}

		// The isRunnable() method uses an exchange on an atomic
		// boolean initialize to true in order to check if this fiber
		// is runnable.
		if (isRunnable(fiber)) {
#ifdef EMPER_STATS
			LawsStrategy::FiberSource fiberSource =
					static_cast<LawsStrategy::FiberSource>(fiber->getFlag());
			switch (fiberSource) {
				case LawsStrategy::FiberSource::fromPriority:
					lawsStrategy.dispatchedFiberFromPriority.fetch_add(1, std::memory_order_relaxed);
					break;
				case LawsStrategy::FiberSource::fromLocal:
					lawsStrategy.dispatchedFiberFromLocal.fetch_add(1, std::memory_order_relaxed);
					break;
				case LawsStrategy::FiberSource::stolen:
					lawsStrategy.dispatchedFiberStolen.fetch_add(1, std::memory_order_relaxed);
					break;
				case LawsStrategy::FiberSource::mainThread:
					lawsStrategy.dispatchedFiberFromMainThread.fetch_add(1, std::memory_order_relaxed);
					break;
				default:
					DIE_MSG("Unknown fiber flag: " << flag);
					break;
			}
#endif
			// The fiber was marked das runnable. Run it now.
			dispatch(fiber);
			// Update the affinity if one was set.
			workeraffinity_t* const affinity = getAffinityBuffer(fiber);
			if (affinity) *affinity = Runtime::getWorkerId();
		}

		if (decreaseRefCount(fiber) == 0) {
			recycle(fiber);
		}
	}
}
