#pragma once

#include "Fiber.hpp"
#include "Scheduler.hpp"
#include "emper-config.h"
#include "lib/adt/LockedQueue.hpp"
#include "lib/adt/LockedUnboundedQueue.hpp"
#include "lib/adt/MpscQueue.hpp"
#include "lib/adt/WsClQueue.hpp"

class LawsStrategy;

class LawsScheduler : public Scheduler {
	template <size_t SIZE>
#ifdef EMPER_LOCKED_WS_QUEUE
	using WsQueue = adt::LockedQueue<Fiber*, SIZE>;
#else
	using WsQueue = adt::WsClQueue<Fiber*, SIZE>;
#endif

	typedef
#ifdef EMPER_LOCKED_MPSC_QUEUE
			adt::LockedUnboundedQueue
#else
			adt::MpscQueue
#endif
			<Fiber>
					LawsMpscQueue;

 public:
	static const int QUEUE_SIZE = 1024;

 private:
	LawsMpscQueue** priorityQueues;

	WsQueue<QUEUE_SIZE>** queues;

	static thread_local LawsMpscQueue priorityQueue;

	static thread_local LawsScheduler::WsQueue<QUEUE_SIZE> queue;

	WsQueue<QUEUE_SIZE>* mainThreadQueue;

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wattributes"
	LawsStrategy& lawsStrategy
#ifndef EMPER_STATS
			ATTR_UNUSED
#endif
			;
#pragma GCC diagnostic pop

 public:
	LawsScheduler(Runtime& runtime, LawsStrategy& lawsStrategy);

	void schedule(Fiber& fiber) override;

	Fiber* nextFiber() override;
};
