#pragma once

#include <atomic>
#include <cstdint>
#include <memory>

#include "LawsDispatcher.hpp"
#include "LawsScheduler.hpp"
#include "LawsStrategyStats.hpp"
#include "RuntimeStrategy.hpp"

class LawsScheduler;
class LawsDispater;

class LawsStrategy : public RuntimeStrategy {
 private:
	enum struct FiberSource : unsigned int {
		fromPriority,
		fromLocal,
		stolen,
		mainThread,
	};

	std::atomic<std::uint64_t> scheduledFibersToRemotePriority;
	std::atomic<std::uint64_t> scheduledFibersToLocal;
	std::atomic<std::uint64_t> dispatchedFiberFromPriority;
	std::atomic<std::uint64_t> dispatchedFiberFromLocal;
	std::atomic<std::uint64_t> dispatchedFiberStolen;
	std::atomic<std::uint64_t> dispatchedFiberFromMainThread;

	LawsStrategy()
			: scheduledFibersToRemotePriority(0),
				scheduledFibersToLocal(0),
				dispatchedFiberFromPriority(0),
				dispatchedFiberFromLocal(0),
				dispatchedFiberStolen(0),
				dispatchedFiberFromMainThread(0) {}

	Scheduler& getScheduler(Runtime& runtime);

	Dispatcher& getDispatcher(Runtime& runtime);

 public:
	virtual std::shared_ptr<RuntimeStrategyStats> getStats();

	static LawsStrategy INSTANCE;

	friend LawsScheduler;
	friend LawsDispatcher;
	friend LawsStrategyStats;
};
