#include <chrono>
#include <mutex>
#include <thread>

#include "BinaryPrivateSemaphore.hpp"
#include "Runtime.hpp"

static std::chrono::nanoseconds threadTimeToSpawn() {
	std::chrono::time_point<std::chrono::high_resolution_clock> end;
	auto start = std::chrono::high_resolution_clock::now();
	std::thread t([&end] { end = std::chrono::high_resolution_clock::now(); });
	t.join();

	return std::chrono::duration_cast<std::chrono::nanoseconds>(end - start);
}

static std::chrono::nanoseconds fiberTimeToSpawn() {
	std::chrono::nanoseconds res;
	Runtime runtime;

	std::mutex testFinished;
	testFinished.lock();

	Fiber* fiber = Fiber::from([&runtime, &res, &testFinished] {
		BPS bps;
		std::chrono::time_point<std::chrono::high_resolution_clock> end;
		Fiber* fiber = Fiber::from([&end, &bps] {
			end = std::chrono::high_resolution_clock::now();
			bps.signal();
		});

		auto start = std::chrono::high_resolution_clock::now();
		runtime.schedule(*fiber);

		bps.wait();
		res = std::chrono::duration_cast<std::chrono::nanoseconds>(end - start);
		testFinished.unlock();
	});
	runtime.schedule(*fiber);

	testFinished.lock();

	// The destructor of Runtime causes all runtime threads to terminate.
	return res;
}

int main(UNUSED_ARG int argc, UNUSED_ARG char* argv[]) {
	{
		auto ttts = threadTimeToSpawn();
		std::cout << "Thread time to spawn: " << ttts.count() << " ns" << std::endl;
	}

	{
		auto ftts = fiberTimeToSpawn();
		std::cout << "Fiber time to spawn: " << ftts.count() << " ns" << std::endl;
	}
}
